{% extends "organization/read_base.html" %}

{% block subtitle %}{{ _('Activity Stream') }} - {{ super() }}{% endblock %}

{% block primary_content_inner %}
  <h2 class="hide-heading">{% block page_heading %}{{ _('Activity Stream') }}{% endblock %}</h2>
  {% block activity_stream %}
    {{ c.group_activity_stream | safe }}
  {% endblock %}
{% endblock %}
