{
  "": {
    "domain": "ckan", 
    "lang": "eu", 
    "plural-forms": "nplurals=2; plural=(n != 1);"
  }, 
  "Could not load view": [
    null, 
    "Bista ez da kargatu"
  ], 
  "DataProxy returned an error": [
    null, 
    "DataProxyk akats bat bidali du"
  ], 
  "DataStore returned an error": [
    null, 
    "DataStorek akats bat bidali du"
  ], 
  "Filters": [
    null, 
    "Iragazkiak"
  ], 
  "Graph": [
    null, 
    "Grafikoa"
  ], 
  "Grid": [
    null, 
    "Taula"
  ], 
  "Image": [
    null, 
    "Irudia"
  ], 
  "Input is too short, must be at least one character": [
    "Input is too short, must be at least %(num)d characters", 
    "", 
    ""
  ], 
  "Loading...": [
    null, 
    "Kargatzen ..."
  ], 
  "Map": [
    null, 
    "Mapa"
  ], 
  "error loading view": [
    null, 
    "akatsa bista kargatzean"
  ]
}