.group .media-vertical .image {
  margin: 0 -5px 5px;
}

// Group List

// A group item summary plus a list of datasets belonging to the group.

// TODO: This should really be on the parent.
.group-list:nth-child(odd) {
  clear: left;
}

.group-list .module-heading {
  padding-top: 15px;
  padding-bottom: 15px;
}

.group-list .dataset-content {
  min-height: 54px;
}

.group-list .module-heading h3 {
  margin-bottom: 2px;
}

.group-list .module-heading h3 a {
  color: @listingHeadingTextColor;
}

.group-list .module-heading .media-image {
  overflow: hidden;
  max-height: 60px;
  img {
    max-width: 85px;
  }
}
