{
  "": {
    "domain": "ckan",
    "lang": "fa_IR",
    "plural-forms": "nplurals=2; plural=(n > 1);"
  },
  "An Error Occurred": [
    null,
    "اشتباهی پیش آمده"
  ],
  "An error occurred: %(text)s %(error)s": [
    null,
    "ایرادی پیش آمده: %(text)s %(error)s"
  ],
  "Are you sure you want to perform this action?": [
    null,
    "آیا مطمئن‌اید که می‌خواهید این کار را انجام دهید؟"
  ],
  "Cancel": [
    null,
    "لغو"
  ],
  "Confirm": [
    null,
    "تأیید"
  ],
  "Edit": [
    null,
    "ویرایش"
  ],
  "Failed to load data API information": [
    null,
    "بارگذاری اطلاعات داده API ناموفق بود"
  ],
  "File": [
    null,
    "فایل"
  ],
  "Follow": [
    null,
    "دنبال کردن"
  ],
  "Hide": [
    null,
    "پنهان کردن موارد"
  ],
  "Image": [
    null,
    "تصویر"
  ],
  "Input is too short, must be at least one character": [
    null,
    "پارامتر وارد شده بیش از حد کوتاه است، و باید حداقل یک حرف باشد"
  ],
  "Link": [
    null,
    "لینک"
  ],
  "Link to a URL on the internet (you can also link to an API)": [
    null,
    "به یک URL روی اینترنت لینک بدهید (می‌توانید به یک API هم لینک بدهید)"
  ],
  "Loading...": [
    null,
    "در حال بارگذاری ..."
  ],
  "No matches found": [
    null,
    "همتایی پیدا نشد"
  ],
  "Please Confirm Action": [
    null,
    "لطفا اقدام مورد نظر را تأیید کنید"
  ],
  "Remove": [
    null,
    "حذف "
  ],
  "Reorder resource view": [
    null,
    "سفارش مجدد نمایش منبع"
  ],
  "Reorder resources": [
    null,
    "منابع را دوباره سفارش دهید"
  ],
  "Resource uploaded": [
    null,
    "منبع آپلود شده است"
  ],
  "Save order": [
    null,
    "ذخیره کردن سفارش"
  ],
  "Saving...": [
    null,
    "در حال ذخیره ..."
  ],
  "Show more": [
    null,
    "دیدن موارد بیشتر"
  ],
  "Start typing…": [
    null,
    "تایپ کردن را شروع کنید ..."
  ],
  "There are unsaved modifications to this form": [
    null,
    "بعضی تغییرات اعمال شده به این فرم ذخیره نشده‌اند"
  ],
  "There is no API data to load for this resource": [
    null,
    "هیچ داده API را نمی‌توان برای این منبع بارگذاری کرد"
  ],
  "URL": [
    null,
    "URL"
  ],
  "Unable to authenticate upload": [
    null,
    "تأیید اعتبار آپلود ممکن نشد"
  ],
  "Unable to get data for uploaded file": [
    null,
    "دریافت داده لازم برای آپلود فایل ممکن نشد"
  ],
  "Unable to upload file": [
    null,
    "آپلود فایل ممکن نیست"
  ],
  "Unfollow": [
    null,
    "دنبال نکردن"
  ],
  "Upload": [
    null,
    "آپلود"
  ],
  "Upload a file": [
    null,
    "آپلود کردن فایل"
  ],
  "Upload a file on your computer": [
    null,
    "آپلود یک فایل روی کامپیوترتان"
  ],
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null,
    "شما در حال آپلود یک فایل هستید. آیا مطمئن‌اید می‌خواهید این صفحه را ترک، و آپلود را متوقف کنید."
  ],
  "error loading view": [
    null,
    "خطا در بارگذاری نمایش"
  ]
}