{
  "": {
    "domain": "ckan",
    "lang": "he",
    "plural-forms": "nplurals=4; plural=(n == 1 && n % 1 == 0) ? 0 : (n == 2 && n % 1 == 0) ? 1: (n % 10 == 0 && n % 1 == 0 && n > 10) ? 2 : 3;"
  },
  "An Error Occurred": [
    null,
    "אירעה שגיאה"
  ],
  "Are you sure you want to perform this action?": [
    null,
    "האם אתם בטוחים שברצונכם לבצע את הפעולה הזו?"
  ],
  "Cancel": [
    null,
    "ביטול"
  ],
  "Confirm": [
    null,
    "אישור"
  ],
  "Edit": [
    null,
    "לערוך"
  ],
  "Failed to load data API information": [
    null,
    "לא הצלחתי לטעון את נתוני המידע של ה-API "
  ],
  "File": [
    null,
    "קובץ"
  ],
  "Follow": [
    null,
    "עקוב"
  ],
  "Hide": [
    null,
    "החביאי"
  ],
  "Image": [
    null,
    "תמונה"
  ],
  "Input is too short, must be at least one character": [
    null,
    "הקלט קצר מידי. צריך להכניס לפחות תו אחד"
  ],
  "Link": [
    null,
    "קישור"
  ],
  "Link to a URL on the internet (you can also link to an API)": [
    null,
    "קישור לכתובת אינטרנט (ניתן לקשר גם ל-API)"
  ],
  "Loading...": [
    null,
    "טוען..."
  ],
  "No matches found": [
    null,
    "לא נמצאו התאמות"
  ],
  "Please Confirm Action": [
    null,
    "אשר את הפעולה בבקשה"
  ],
  "Remove": [
    null,
    "הסרה"
  ],
  "Reorder resources": [
    null,
    "סדר מחדש משאבים"
  ],
  "Resource uploaded": [
    null,
    "המקור הועלה"
  ],
  "Save order": [
    null,
    "סדר שמירה"
  ],
  "Saving...": [
    null,
    "שומר..."
  ],
  "Show more": [
    null,
    "הראי עוד"
  ],
  "Start typing…": [
    null,
    "התחילו להקליד..."
  ],
  "There are unsaved modifications to this form": [
    null,
    "ישנם שינויים שלא נשמרו לטופס הזה"
  ],
  "There is no API data to load for this resource": [
    null,
    "אין נתוני ממשק מכונה כדי לטעון את המקור הזה"
  ],
  "URL": [
    null,
    "URL"
  ],
  "Unable to authenticate upload": [
    null,
    "איני יכולה לאמת את ההעלאה"
  ],
  "Unable to get data for uploaded file": [
    null,
    "איני מצליחה לקבל נתונים מהקובץ שהועלה"
  ],
  "Unable to upload file": [
    null,
    "איני יכולה להעלות את הקובץ"
  ],
  "Unfollow": [
    null,
    "ביטול עקיבה"
  ],
  "Upload": [
    null,
    "העלה"
  ],
  "Upload a file": [
    null,
    "העלאת קבצים"
  ],
  "Upload a file on your computer": [
    null,
    "העלו תמונה מהמחשב שלכם"
  ],
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null,
    "אתם מעלים קובץ. האם אתם בטוחים שברצונכם לצאת מהעמוד ולהפסיק את ההעלאה?"
  ]
}