{% extends "group/edit_base.html" %}

{% block page_primary_action %}{% endblock %}

{% block breadcrumb_content %}
  <li>{{ h.nav_link(_('Groups'), controller='group', action='index') }}</li>
  <li class="active">{% block breadcrumb_link %}{{ h.nav_link(_('Add a Group'), controller='group', action='new') }}{% endblock %}</li>
{% endblock %}

{% block primary_content_inner %}
  <h1 class="{% block page_heading_class %}page-heading{% endblock %}">{% block page_heading %}{{ _('Group Form') }}{% endblock %}</h1>
  {% block form %}
    {{ c.form | safe }}
  {% endblock %}
{% endblock %}
