{% extends "group/snippets/group_form.html" %}

{#
As the form is rendered as a seperate page we take advantage of this by
overriding the form blocks depending on the current context
#}
{% block dataset_fields %}
  {% if action == "edit" %}{{ super() }}{% endif %}
{% endblock %}

{% block custom_fields %}
  {% if action == "edit" %}{{ super() }}{% endif %}
{% endblock %}

{% block save_text %}
  {%- if action == "edit" -%}
    {{ _('Update Group') }}
  {%- else -%}
    {{ _('Create Group') }}
  {%- endif -%}
{% endblock %}

{% block delete_button %}
  {% if action == "edit" %}{{ super() }}{% endif %}
{% endblock %}
