{#
render a list of tags linking to the dataset search page
tags: list of tags
#}
{% set _class = _class or 'tag-list' %}
{% block tag_list %}
  <ul class="{{ _class }}">
    {% for tag in tags %}
      <li>
        <a class="{% block tag_list_item_class %}tag{% endblock %}" href="{% url_for controller='package', action='search', tags=tag.name %}">{{ h.truncate(tag.display_name, 22) }}</a>
      </li>
    {% endfor %}
  </ul>
{% endblock %}
