{
  "": {
    "domain": "ckan",
    "lang": "cs_CZ",
    "plural-forms": "nplurals=4; plural=(n == 1 && n % 1 == 0) ? 0 : (n >= 2 && n <= 4 && n % 1 == 0) ? 1: (n % 1 != 0 ) ? 2 : 3;"
  },
  "An Error Occurred": [
    null,
    "Nastala chyba"
  ],
  "An error occurred: %(text)s %(error)s": [
    null,
    "Stala se chyba: %(text)s %(error)s"
  ],
  "Are you sure you want to perform this action?": [
    null,
    "Jste si jistí, že chcete provést tuto akci?"
  ],
  "Cancel": [
    null,
    "Zrušit"
  ],
  "Confirm": [
    null,
    "Potvrdit"
  ],
  "Edit": [
    null,
    "Upravit"
  ],
  "Failed to load data API information": [
    null,
    "Pokus o získání informací pomocí API selhal"
  ],
  "File": [
    null,
    "Soubor"
  ],
  "Follow": [
    null,
    "Sledovat"
  ],
  "Hide": [
    null,
    "Skrýt"
  ],
  "Image": [
    null,
    "Obrázek"
  ],
  "Input is too short, must be at least one character": [
    null,
    "Zadaný vstup je příliš krátký, musíte zadat alespoň jeden znak"
  ],
  "Link": [
    null,
    "Odkaz"
  ],
  "Link to a URL on the internet (you can also link to an API)": [
    null,
    "Odkaz na internetovou URL adresu (můžete také zadat odkaz na API)"
  ],
  "Loading...": [
    null,
    "Nahrávám..."
  ],
  "No matches found": [
    null,
    "Nenalezena žádná shoda"
  ],
  "Please Confirm Action": [
    null,
    "Prosím potvrďte akci"
  ],
  "Remove": [
    null,
    "Odstranit"
  ],
  "Reorder resource view": [
    null,
    "Změnit pořadí zobrazení datového zdroje"
  ],
  "Reorder resources": [
    null,
    "Změnit pořadí zdrojů"
  ],
  "Resource uploaded": [
    null,
    "Datový zdroj nahrán"
  ],
  "Save order": [
    null,
    "Uložit pořadí"
  ],
  "Saving...": [
    null,
    "Ukládám..."
  ],
  "Show more": [
    null,
    "Ukázat více"
  ],
  "Start typing…": [
    null,
    "Začněte psát..."
  ],
  "There are unsaved modifications to this form": [
    null,
    "Tento formulář obsahuje neuložené změny"
  ],
  "There is no API data to load for this resource": [
    null,
    "Tento datový zdroj neobsahuje žádná data, která lze poskytnou přes API"
  ],
  "URL": [
    null,
    "URL"
  ],
  "Unable to authenticate upload": [
    null,
    "Nastala chyba autentizace při nahrávání dat"
  ],
  "Unable to get data for uploaded file": [
    null,
    "Nelze získat data z nahraného souboru"
  ],
  "Unable to upload file": [
    null,
    "Nelze nahrát soubor"
  ],
  "Unfollow": [
    null,
    "Přestat sledovat"
  ],
  "Upload": [
    null,
    "Nahrát"
  ],
  "Upload a file": [
    null,
    "Nahrát soubor"
  ],
  "Upload a file on your computer": [
    null,
    "Nahrát soubor z počítače"
  ],
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null,
    "Právě nahráváte soubor. Jste si opravdu jistí, že chcete tuto stránku opustit a ukončit tak nahrávání?"
  ],
  "error loading view": [
    null,
    "Chyba při načítání zobrazení"
  ]
}