{
  "": {
    "domain": "ckan",
    "lang": "fi",
    "plural-forms": "nplurals=2; plural=(n != 1);"
  },
  "An Error Occurred": [
    null,
    "Tapahtui virhe"
  ],
  "Are you sure you want to perform this action?": [
    null,
    "Oletko varma, että haluat tehdä tämän?"
  ],
  "Cancel": [
    null,
    "Peruuta"
  ],
  "Confirm": [
    null,
    "Vahvista"
  ],
  "Edit": [
    null,
    "Muokkaa"
  ],
  "Failed to load data API information": [
    null,
    "Data API:n tietojen lataaminen epäonnistui"
  ],
  "File": [
    null,
    "Tiedosto"
  ],
  "Follow": [
    null,
    "Seuraa"
  ],
  "Hide": [
    null,
    "Piilota"
  ],
  "Image": [
    null,
    "Kuva"
  ],
  "Input is too short, must be at least one character": [
    null,
    "Liian lyhyt, pitää olla vähintään yksi merkki"
  ],
  "Link": [
    null,
    "Linkki"
  ],
  "Link to a URL on the internet (you can also link to an API)": [
    null,
    "Linkki URL-osoitteeseen internetissä (voit myös linkittää API:in)"
  ],
  "Loading...": [
    null,
    "Ladataan..."
  ],
  "No matches found": [
    null,
    "Ei yhtään osumaa"
  ],
  "Please Confirm Action": [
    null,
    "Vahvista toimenpide"
  ],
  "Remove": [
    null,
    "Poista"
  ],
  "Reorder resource view": [
    null,
    "Uudelleenjärjestä resurssinäyttö"
  ],
  "Reorder resources": [
    null,
    "Uudelleenjärjestä resurssit"
  ],
  "Resource uploaded": [
    null,
    "Resurssi on ladattu"
  ],
  "Save order": [
    null,
    "Tallenna järjestys"
  ],
  "Saving...": [
    null,
    "Tallentaa..."
  ],
  "Show more": [
    null,
    "Näytä enemmän"
  ],
  "Start typing…": [
    null,
    "Ala kirjoittaa..."
  ],
  "There are unsaved modifications to this form": [
    null,
    "Lomakkeella on tallentamattomia tietoja"
  ],
  "There is no API data to load for this resource": [
    null,
    "API:n kautta ei ole ladattavissa dataa tälle resurssille"
  ],
  "URL": [
    null,
    "URL"
  ],
  "Unable to authenticate upload": [
    null,
    "Lataamisen autentikointi epäonnistui"
  ],
  "Unable to get data for uploaded file": [
    null,
    "Datan saaminen ladattavaan tiedostoon epäonnistui"
  ],
  "Unable to upload file": [
    null,
    "Tiedoston lataaminen epäonnistui"
  ],
  "Unfollow": [
    null,
    "Lopeta seuraaminen"
  ],
  "Upload": [
    null,
    "Lataa"
  ],
  "Upload a file": [
    null,
    "Tallenna tiedosto"
  ],
  "Upload a file on your computer": [
    null,
    "Lataa tiedosto työasemalta"
  ],
  "You are uploading a file. Are you sure you want to navigate away and stop this upload?": [
    null,
    "Olet parhaillaan lataamassa tiedostoa palvelimelle. Oletko varma, että haluat poistua sivulta ja keskeyttää lataamisen?"
  ]
}